# Environment

- **Current Date**: {{now.strftime("%A, %B %d, %Y")}}
- **Operating System**: {{os_info}}
- **Working Directory**: {{cwd}}
- **Shell**: {{shell_info}}

The user has granted you access to run tools in service of their request. Use them when needed.