from setuptools import setup, find_packages

setup(
    name='pegasus-ai',
    version='0.1.6',
    description='A lightweight terminal agent framework for long-running modular agents',
    url='https://github.com/asuzukosi/pegasus',
    author='Asuzu Kosi',
    author_email='keloasuzu@yahoo.com',
    license='BSD 2-clause',
    packages=find_packages(exclude=['tests*', 'docs*', 'assets*', 'pegasus.core*']),
    include_package_data=True,
    package_data={'pegasus': ['prompts/*.j2']},
    install_requires=[
        'art',
        'click',
        'ddgs',
        'fastmcp',
        'jinja2',
        'openai',
        'opencv-python',
        'pillow',
        'platformdirs',
        'playwright',
        'pydantic',
        'python-dotenv',
        'retry',
        'rich',
        'tiktoken',
        'tomli',
    ],
    entry_points={
        'console_scripts': [
            'pegasus-cli=pegasus.cli.cli:run_cli',
        ],
    },
    python_requires='>=3.9',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
)
